--[[
################################################################
#
#	Hunter Cookies by Mosphet <Inner Sanctum> @ Silvermoon EU
#	Dude I need for use! WTF PRIST!
#
################################################################
]]

local addon = LibStub("AceAddon-3.0"):NewAddon("MosphetCookies", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
_G.MC = addon
addon.version = tonumber(("$Revision: 7 $"):match("%d+"))
addon.callbacks = LibStub("CallbackHandler-1.0"):New(addon)
addon.media = LibStub("LibSharedMedia-3.0")

-- -----------------
--  Locals
-- -----------------
local _G, type, tonumber, tostring = _G, type, tonumber, tostring;
local tinsert = tinsert;
local format, strsub, strfind, gsub = format, string.sub, string.find, string.gsub;
local ipairs = ipairs;
local min = math.min;

local class, _ = UnitClass("player")
if class == "Hunter" then 
	enable = true
else
	return 
end

local db
local debug = nil
local defaults = {
	profile = {
		-- -------------
		-- General
		-- -------------
		enabled = true,
		fontface = "Lucidia Sans",
		fontsize = 14,
		justification = "RIGHT",
		-- -------------
		-- Displays
		-- -------------
		aimed = true,
		steady = true,
		cobra = true,
		hastev = nil,
		hastep = nil,
		petap = nil,
		rabid = nil,
		-- -------------
		-- Thresholds
		-- -------------
		--aisthresh = 1.7,
		-- -------------
		-- Positions
		-- -------------
		xOfs = 0,
		yOfs = 0,
	},
}

-- -----------------
--  Derps
-- -----------------
local mediapath = "Interface\\AddOns\\MosphetCookies\\media\\"
local fontlol = mediapath.."fonts\\lsans.ttf"
local fontlol2 = mediapath.."fonts\\myriad.otf"
local fsize = 14
local refresh = 1

function addon:print(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00[MC]|r "..msg)
end

function addon:debug(msg)
	if(debug) then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00[MC Debug]|r "..msg)
	end
end

function addon:error(s, l)
	-- ehh ye. useful wrapper
	error(s, l)
end

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("MosphetCookiesDB", defaults, "Default")
	db = self.db.profile
	
	if not self.Awsumness then
		self:CreateAwsumness()
	end
	
	self:RegisterChatCommand("mc", "SlashHandler");	
	self:RegisterChatCommand("mosphet", "SlashHandler");	
	self:RegisterChatCommand("cookies", "SlashHandler");
end
	
function addon:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeftCombat")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateCast");
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "TalentCheck");
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "TalentCheck");
end

function addon:TalentCheck()
	if not GetTalentTabInfo(1) then
		talents = "Talents not loaded yet!"
		return -- Talents aren't loaded yet!
	end

	local specID = GetPrimaryTalentTree()
	if specID then
		local _, specName = GetTalentTabInfo(specID)
		self:debug(string.format("%s %s", specName, UnitClass("player")))
		talents = specName
	else
		self:debug("Unable to detect spec")
	end
end

function addon:OnDisabled()
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_ENTERING_WORLD");
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
end

function addon:EnteringCombat()
	addon:TalentCheck()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "UpdateCast");
	refreshcast = GetTime();
	self:debug("Entering Combat");
	self:debug("refreshcast: "..refreshcast);
	self:debug("Talents: "..talents);
end

function addon:LeftCombat()
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self:debug("Left Combat");
end

function addon:UpdateCast()
	local line1, line2, line3, line4, line5, line6, line7 = "\n","\n","\n","\n","\n","\n","\n";
	if refreshcast > GetTime() then return end
	self:debug("Checking Cast");
	if talents == "Marksmanship" then
		if db.aimed then
			local _, _, _, _, _, _, castTime = GetSpellInfo("Aimed Shot");
			local aimed = min(castTime/1000)
			line1 = "AiS: "..aimed.."\n";
		end
		if db.steady then
			local _, _, _, _, _, _, castTime = GetSpellInfo("Steady Shot");
			local steady = min(castTime/1000)
			line2 = "SS: "..steady.."\n";
		end
	else
		if db.cobra then
			local _, _, _, _, _, _, castTime = GetSpellInfo("Cobra Shot");
			local cobra = min(castTime/1000)
			line3 = "Cobra: "..cobra.."\n";
		end
	end
	if db.hastev then
		rating = GetCombatRating(CR_HASTE_RANGED);
		line4 = "Haste: "..rating.."\n";
	end
	if db.hastep then
		percent = GetRangedHaste();
		line5 = "Haste: "..format("%.2F%%", percent).."\n";
	end
	if db.petap then
		local base, posBuff, negBuff = UnitAttackPower("pet");
		local petap = base+posBuff+negBuff;
		line6 = "Pet AP: "..petap.."\n";
	end
	if db.rabid then
		local _, _, _, count = UnitAura("pet", "Rabid Power");
		if count then
			line7 = "Rabid: "..count.."\n";
		else
			line7 = "Rabid: 0\n";
		end
	end
	MosphetCookies.CenterText:AddMessage(line1..line2..line3..line4..line5..line6..line7);
	refreshcast = GetTime() + refresh
end

function addon:UpdateJustification(v)
	MosphetCookies.CenterText:SetJustifyH(v);
end

function addon:UpdateFont()
	fontface = db.fontface
	fontsize = db.fontsize
	MosphetCookies.CenterText:SetFont(fontlol, fontsize, "OUTLINE")
end

-- Gief frame plz ;o
function addon:CreateAwsumness()
	do
		local anchor = CreateFrame("Frame", nil, UIParent)
		anchor:SetSize(200, 15)
		anchor:ClearAllPoints()
		anchor:SetPoint("CENTER", UIParent, "CENTER", db.xOfs, db.yOfs)
		anchor:SetFrameStrata("LOW")
		anchor:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",
		insets = {left = 0,right = 0,top = 0,bottom = 0}});
		anchor:SetBackdropColor(0, 0, 0, .5)
		anchor:EnableMouse(true)
		anchor:SetScript("OnLeave", function(self)
			-- save pos
			local _, _, _, xOfs, yOfs = self:GetPoint()
			db.xOfs = xOfs
			db.yOfs = yOfs
		end)
		
		anchor.tr = anchor:CreateTitleRegion()
		anchor.tr:SetAllPoints(anchor)
		
		local mf = CreateFrame("MessageFrame", "MosphetCookies", UIParent)
		mf:SetSize(200,26)
		mf:ClearAllPoints()
		mf:SetPoint("TOP", anchor, "TOP", 0, -16)
		mf:Show()
		mf:SetFont(fontlol, fsize, "OUTLINE")
		mf:SetShadowOffset(0, 0)
		mf:SetJustifyH(db.justification)
		mf:SetJustifyV("TOP")
		mf:SetTimeVisible(2)
		mf:SetFadeDuration(.5)
		
		mf.anchor = anchor
		
		mf.anchor:Hide()
		MosphetCookies.CenterText = mf;
	end
end

function addon:ResetAnchor()
	db.xOfs = 0
	db.yOfs = 0
	MosphetCookies.anchor:SetPoint("CENTER", UIParent, "CENTER", db.xOfs, db.yOfs)
end

function addon:ToggleAnchor()
	if MosphetCookies.CenterText.anchor:IsShown() then
		MosphetCookies.CenterText.anchor:Hide()
		MosphetCookies.CenterText:SetTimeVisible(2)
	else
		MosphetCookies.CenterText.anchor:Show()
		MosphetCookies.CenterText:SetTimeVisible(120)
		MosphetCookies.CenterText:AddMessage("AiS: 1.234\nSS: 1.234\nCoS: 1.234\nHaste: 1234\nHaste: 12.34%\nPet AP: 1234\nRabid: 0", 0.6, 0.7, 1, 1)
	end
end

function addon:SlashHandler(command)
	command = self:Explode(" ", string.lower(command))
	if(command[1] == "lock") then
		self:ToggleAnchor()
	elseif(command[1] == "reset") then
		self:ResetAnchor()
		self:print("Position Reset!")
	elseif(command[1] == "debug") then
		if (debug) then
			debug = nil
		else
			debug = true
		end
	elseif(command[1] == "help") then
		self:print("|cFFFFCC00MosphetCookies For Retards:")
		self:print("|cFFCC9900   lock|r - Toggle the anchor lock")
		self:print("|cFFCC9900   reset|r - Resets the frame position")
		self:print("|cFFCC9900   debug|r - Enable/Disable Debug")
	else
		self.Options:ToggleConfig()
	end
end

function addon:Explode(d, str)
	local t, ll
	t={}
	ll=0
	if(#str == 1) then return str end
		while true do
			l=strfind(str, d, ll+1, true)
			if l~=nil then
				tinsert(t, strsub(str, ll, l-1))
				ll=l+1
			else
				tinsert(t, strsub(str,ll))
				break
			end
		end
	return t
end